# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""


import pprint
import re  # noqa: F401
from typing import TYPE_CHECKING

import six

from grid.openapi.configuration import Configuration

if TYPE_CHECKING:
    from datetime import datetime
    from grid.openapi.models import *

class V1GetUserResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'api_key': 'str',
        'email': 'str',
        'first_name': 'str',
        'id': 'str',
        'last_name': 'str',
        'picture_url': 'str',
        'status': 'V1UserStatus',
        'username': 'str'
    }

    attribute_map = {
        'api_key': 'apiKey',
        'email': 'email',
        'first_name': 'firstName',
        'id': 'id',
        'last_name': 'lastName',
        'picture_url': 'pictureUrl',
        'status': 'status',
        'username': 'username'
    }

    def __init__(self, api_key: 'str' = None, email: 'str' = None, first_name: 'str' = None, id: 'str' = None, last_name: 'str' = None, picture_url: 'str' = None, status: 'V1UserStatus' = None, username: 'str' = None, _configuration=None):  # noqa: E501
        """V1GetUserResponse - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._api_key = None
        self._email = None
        self._first_name = None
        self._id = None
        self._last_name = None
        self._picture_url = None
        self._status = None
        self._username = None
        self.discriminator = None

        if api_key is not None:
            self.api_key = api_key
        if email is not None:
            self.email = email
        if first_name is not None:
            self.first_name = first_name
        if id is not None:
            self.id = id
        if last_name is not None:
            self.last_name = last_name
        if picture_url is not None:
            self.picture_url = picture_url
        if status is not None:
            self.status = status
        if username is not None:
            self.username = username

    @property
    def api_key(self) -> 'str':
        """Gets the api_key of this V1GetUserResponse.  # noqa: E501


        :return: The api_key of this V1GetUserResponse.  # noqa: E501
        :rtype: str
        """
        return self._api_key

    @api_key.setter
    def api_key(self, api_key: 'str'):
        """Sets the api_key of this V1GetUserResponse.


        :param api_key: The api_key of this V1GetUserResponse.  # noqa: E501
        :type: str
        """

        self._api_key = api_key

    @property
    def email(self) -> 'str':
        """Gets the email of this V1GetUserResponse.  # noqa: E501


        :return: The email of this V1GetUserResponse.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email: 'str'):
        """Sets the email of this V1GetUserResponse.


        :param email: The email of this V1GetUserResponse.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def first_name(self) -> 'str':
        """Gets the first_name of this V1GetUserResponse.  # noqa: E501


        :return: The first_name of this V1GetUserResponse.  # noqa: E501
        :rtype: str
        """
        return self._first_name

    @first_name.setter
    def first_name(self, first_name: 'str'):
        """Sets the first_name of this V1GetUserResponse.


        :param first_name: The first_name of this V1GetUserResponse.  # noqa: E501
        :type: str
        """

        self._first_name = first_name

    @property
    def id(self) -> 'str':
        """Gets the id of this V1GetUserResponse.  # noqa: E501


        :return: The id of this V1GetUserResponse.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id: 'str'):
        """Sets the id of this V1GetUserResponse.


        :param id: The id of this V1GetUserResponse.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def last_name(self) -> 'str':
        """Gets the last_name of this V1GetUserResponse.  # noqa: E501


        :return: The last_name of this V1GetUserResponse.  # noqa: E501
        :rtype: str
        """
        return self._last_name

    @last_name.setter
    def last_name(self, last_name: 'str'):
        """Sets the last_name of this V1GetUserResponse.


        :param last_name: The last_name of this V1GetUserResponse.  # noqa: E501
        :type: str
        """

        self._last_name = last_name

    @property
    def picture_url(self) -> 'str':
        """Gets the picture_url of this V1GetUserResponse.  # noqa: E501


        :return: The picture_url of this V1GetUserResponse.  # noqa: E501
        :rtype: str
        """
        return self._picture_url

    @picture_url.setter
    def picture_url(self, picture_url: 'str'):
        """Sets the picture_url of this V1GetUserResponse.


        :param picture_url: The picture_url of this V1GetUserResponse.  # noqa: E501
        :type: str
        """

        self._picture_url = picture_url

    @property
    def status(self) -> 'V1UserStatus':
        """Gets the status of this V1GetUserResponse.  # noqa: E501


        :return: The status of this V1GetUserResponse.  # noqa: E501
        :rtype: V1UserStatus
        """
        return self._status

    @status.setter
    def status(self, status: 'V1UserStatus'):
        """Sets the status of this V1GetUserResponse.


        :param status: The status of this V1GetUserResponse.  # noqa: E501
        :type: V1UserStatus
        """

        self._status = status

    @property
    def username(self) -> 'str':
        """Gets the username of this V1GetUserResponse.  # noqa: E501


        :return: The username of this V1GetUserResponse.  # noqa: E501
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username: 'str'):
        """Sets the username of this V1GetUserResponse.


        :param username: The username of this V1GetUserResponse.  # noqa: E501
        :type: str
        """

        self._username = username

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1GetUserResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'V1GetUserResponse') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, V1GetUserResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other: 'V1GetUserResponse') -> bool:
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1GetUserResponse):
            return True

        return self.to_dict() != other.to_dict()



from dataclasses import dataclass
from os.path import split
from pathlib import Path
from typing import Iterator, Optional, Union, List
from re import compile

# use simple import to avoid 'partially initialized module' with experiments + runs
import grid.sdk.experiments
import grid.sdk.runs
from grid.cli.utilities import is_experiment
from grid.sdk.utils.downloader import DownloadableObject, Downloader


@dataclass(frozen=True)
class Artifact:
    url: str
    filename: str
    experiment_name: str
    run_name: str

    def to_downloadable_object(self) -> DownloadableObject:
        """Returns an object that can be used by grid.sdk.utils.downloader.Downloader
        """
        path, filename = split(self.filename)
        return DownloadableObject(
            url=self.url,
            download_path=path,
            filename=filename,
        )

    def download(self, destination: str = "./grid_artifacts"):
        """Downloads the artifact to the filesystem.

        It may be easier to use `experiment.download_artifacts()` or
        `run.download_artifacts()` depending on your use case.

        Parameters
        ----------
        destination : str, optional
            Path to place the download, by default "./grid_artifacts"
        """
        Downloader.single_file_download(
            obj=self.to_downloadable_object(),
            dest=destination,
        )


def _filter_artifacts(artifact_regex: str, artifacts: List[Artifact]) -> List[Artifact]:
    """Extracted from `list_artifacts` for easier unit testing
    """
    regex = compile(artifact_regex)
    filtered = filter(
        lambda artifact: regex.search(artifact.filename),
        artifacts,
    )
    return list(filtered)


def list_artifacts(run_or_experiment_name: str,
                   cluster_id: Optional[str] = None,
                   artifact_regex: str = "") -> List[Artifact]:
    """Lists all artifacts for all experiments created by the user.

    Parameters
    ----------
    run_or_experiment_name : str
        Returns the artifacts for this run / experiment.
    cluster_id : Optional[str], optional
        The id of the cluster to search runs for. If None, the default cluster (as listed
        in settings.json) is used.
    artifact_regex : str, optional
        If provided, only return artifacts which match this regex (applied to artifact's filename)

    Returns
    -------
    List[Artifact]
    """

    # get the artifacts from the appropriate source
    artifacts: Iterator[Artifact]
    if is_experiment(run_or_experiment_name):
        artifacts = grid.sdk.experiments.Experiment(name=run_or_experiment_name, cluster_id=cluster_id).artifacts
    else:
        artifacts = grid.sdk.runs.Run(name=run_or_experiment_name, cluster_id=cluster_id).artifacts

    # filter artifacts if necessary
    return _filter_artifacts(artifact_regex, artifacts)


def download_artifacts(artifacts: Iterator[Artifact], destination: Union[str, Path] = "./grid_artifacts"):
    """Download a list of artifacts in bulk

    Parameters
    ----------
    artifacts : Iterator[Artifact]
        The list of artifacts to download
    destination : Union[str, Path], optional
        Path to download artifacts, by default "./grid_artifacts"
    """
    Downloader.multi_file_download(
        destination=destination,
        downloadable_objects=(a.to_downloadable_object() for a in artifacts),
        description="artifacts downloaded",
        unit="artifact",
    )



from dataclasses import dataclass
import math
import os
import subprocess
import tarfile
from typing import Optional, Tuple

import click

MAX_SPLIT_COUNT = 999


def get_dir_size_and_count(source_dir: str, prefix: Optional[str] = None) -> Tuple[int, int]:
    """
    Get size and file count of a directory

    Parameters
    ----------
    source_dir: str
        Directory path

    Returns
    -------
    Tuple[int, int]
        Size in megabytes and file count
    """
    size = 0
    count = 0
    for root, _, files in os.walk(source_dir, topdown=True):
        for f in files:
            if prefix and not f.startswith(prefix):
                continue

            full_path = os.path.join(root, f)
            size += os.path.getsize(full_path)
            count += 1

    return (size, count)


@dataclass
class TarResults:
    """
    This class holds the results of running tar_path.

    Attributes
    ----------
    before_size: int
        The total size of the original directory files in bytes
    after_size: int
        The total size of the compressed and tarred split files in bytes
    """
    before_size: int
    after_size: int


def get_split_size(
    total_size: int, minimum_split_size: int = 1024 * 1000 * 20, max_split_count: int = MAX_SPLIT_COUNT
) -> int:
    """
    Calculate the split size we should use to split the multipart upload of an object to a bucket.  We are limited
    to 1000 max parts as the way we are using ListMultipartUploads.
    More info
        https://github.com/gridai/grid/pull/5267
        https://docs.aws.amazon.com/AmazonS3/latest/userguide/mpuoverview.html#mpu-process
        https://docs.aws.amazon.com/AmazonS3/latest/API/API_ListMultipartUploads.html
        https://github.com/psf/requests/issues/2717#issuecomment-724725392 Python or requests has a limit of 2**31 bytes for a single file upload.

    Parameters
    ----------
    minimum_split_size: int
        The minimum split size to use
    max_split_count: int
        The maximum split count
    total_size: int
        Total size of the file to split

    Returns
    -------
    int
        Split size
    """
    max_size = max_split_count * (1 << 31)  # max size per part limited by Requests or urllib as shown in ref above
    if total_size > max_size:
        raise click.ClickException(
            f"The size of the datastore to be uploaded is bigger than our {max_size/(1 << 40):.2f} TBytes limit"
        )

    split_size = minimum_split_size
    split_count = math.ceil(total_size / split_size)
    if split_count > max_split_count:
        # Adjust the split size based on max split count
        split_size = math.ceil(total_size / max_split_count)

    return split_size


def tar_path(source_path: str, target_file: str, compression: bool = False) -> TarResults:
    """
    Create tar from directory using `tar`

    Parameters
    ----------
    source_path: str
        Source directory or file
    target_file
        Target tar file
    compression: bool, default False
        Enable compression, which is disabled by default.

    Returns
    -------
    TarResults
        Results that holds file counts and sizes
    """
    if os.path.isdir(source_path):
        before_size, _ = get_dir_size_and_count(source_path)
    else:
        before_size = os.path.getsize(source_path)

    try:
        _tar_path_subprocess(source_path, target_file, compression)
    except subprocess.CalledProcessError:
        _tar_path_python(source_path, target_file, compression)

    after_size = os.stat(target_file).st_size
    return TarResults(before_size=before_size, after_size=after_size)


def _tar_path_python(source_path: str, target_file: str, compression: bool = False) -> None:
    """
    Create tar from directory using `python`

    Parameters
    ----------
    source_path: str
        Source directory or file
    target_file
        Target tar file
    compression: bool, default False
        Enable compression, which is disabled by default.
    """
    file_mode = "w:gz" if compression else "w:"

    with tarfile.open(target_file, file_mode) as tar:
        if os.path.isdir(source_path):
            tar.add(str(source_path), arcname=".")
        elif os.path.isfile(source_path):
            file_info = tarfile.TarInfo(os.path.basename(str(source_path)))
            tar.addfile(file_info, open(source_path))


def _tar_path_subprocess(source_path: str, target_file: str, compression: bool = False) -> None:
    """
    Create tar from directory using `tar`

    Parameters
    ----------
    source_path: str
        Source directory or file
    target_file
        Target tar file
    compression: bool, default False
        Enable compression, which is disabled by default.
    """
    # Only add compression when users explicitly request it.
    # We do this because it takes too long to compress
    # large datastores.
    tar_flags = '-cvf'
    if compression:
        tar_flags = '-zcvf'
    if os.path.isdir(source_path):
        command = f"tar -C {source_path} {tar_flags} {target_file} ./"
    else:
        abs_path = os.path.abspath(source_path)
        parent_dir = os.path.dirname(abs_path)
        base_name = os.path.basename(abs_path)
        command = f"tar -C {parent_dir} {tar_flags} {target_file} {base_name}"

    subprocess.check_call(
        command,
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
        shell=True,
        env={
            "GZIP": "-9",
            "COPYFILE_DISABLE": "1"
        }
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Original solution from
#
#      https://github.com/navdeep-G/setup.py
#
#  Original license notice:
#
#  Copyright 2020 navdeep-G & GitHub contributors
#
#  Permission is hereby granted, free of charge, to any
#  person obtaining a copy of this software and associated
#  documentation files (the "Software"), to deal in the
#  Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute,
#  sublicense, and/or sell copies of the Software, and to
#  permit persons to whom the Software is furnished to do so,
#  subject to the following conditions:
#
#  The above copyright notice and this permission notice shall
#  be included in all copies or substantial portions of the Software.

#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
#  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
#  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.

#  NOTE: To use the 'upload' functionality of this file, you must:
#    $ pip install twine

import os
from pathlib import Path
from shutil import rmtree
import sys
import atexit

from setuptools import Command, find_packages, setup
from setuptools.command.install import install


def read_grid_metadata_file(variable_name: str):
    """Reads the values from the grid metadata.py file without importing it.

    This will read the value associated with a line in the metadata.py file
    which has the format:

        foo_variable = 'bar_value'

    returning the python <string> ``foo_value`` if called with f("foo_variable").

    Note: This will only work for assignments that occur on a single line.
    """
    metadata_file = Path(__file__).parent.joinpath("grid", "metadata.py")
    with metadata_file.open("rt") as f:
        for line in f.readlines():
            if line.startswith(variable_name):
                val = line.split("=", maxsplit=1)[-1]  # don't split at second "=" char in case it is in the value.
                val = val.strip()  # remove leading and trailing whitespace
                val = val.strip("'")  # remove ' char from beginning and end (value is already a string)
                val = val.strip('"')  # remove " char from beginning and end (value is already a string)
                return val
    return ""


#   Package meta-data.
NAME = 'grid'
PACKAGE_NAME = read_grid_metadata_file("__package_name__")
DESCRIPTION = 'Grid Python SDK.'
URL = 'https://grid.ai'
EMAIL = 'grid-eng@grid.ai'
AUTHOR = 'Grid AI Engineering'
REQUIRES_PYTHON = '>=3.8.0'
VERSION = os.getenv("VERSION", read_grid_metadata_file("__version__"))

#  What packages are required for this module to be executed?
REQUIRED = []
with open('requirements.txt') as f:
    for line in f.readlines():
        REQUIRED.append(line.replace('\n', ''))

#  What packages are optional?
EXTRAS = {}

#  The rest you shouldn't have to touch too much :)
#  ------------------------------------------------
#  Except, perhaps the License and Trove Classifiers!
#  If you do change the License, remember to
#  the Trove Classifier for that!

here = os.path.abspath(os.path.dirname(__file__))

#  Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    with open(os.path.join(here, NAME, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION


def check_environment_variables(variables: list) -> bool:
    """
    Check that environment variables have been
    set, raising an error if they have not.

    Parameters
    ----------
    variables: list
        List of environment variables to check.
    Returns
    -------
    bool
        Returns True if all environment variables
        have been set correctly.
    """
    for variable in variables:
        if not os.getenv(variable):
            raise ValueError(f'Environment variable `{variable}` has not been set.')

    return True


class GCPStorage:  # pragma: no cover
    """
    Google Cloud Platform Storage wrapper for uploading
    objects to GCP.
    """
    def __init__(self):
        from google.cloud import storage
        self.storage_client = storage.Client()

    def upload_blob(self, bucket_name, source_file_name, destination_blob_name):
        """Uploads a file to the bucket."""
        bucket = self.storage_client.bucket(bucket_name)
        blob = bucket.blob(destination_blob_name)

        blob.upload_from_filename(source_file_name)

        print("File {} uploaded to {}.".format(source_file_name, destination_blob_name))

    def delete_blob(self, bucket_name, blob_name):
        """Deletes a blob from the bucket."""
        # bucket_name = "your-bucket-name"
        # blob_name = "your-object-name"
        bucket = self.storage_client.bucket(bucket_name)
        blob = bucket.blob(blob_name)
        blob.delete()

        print("Blob {} deleted.".format(blob_name))

    def delete_blobs(self, bucket_name, prefix=''):
        """ Deletes all blobs with a specific prefix from a bucket."""
        bucket = self.storage_client.bucket(bucket_name)
        blobs = bucket.list_blobs()
        for blob in blobs:
            if blob.name.startswith(prefix):
                blob.delete()

    def list_blobs(self, bucket_name, prefix='', delimiter=None):
        """Lists all the blobs in the bucket."""
        blobs = self.storage_client.list_blobs(bucket_name, prefix=prefix, delimiter=delimiter)

        names = []
        for blob in blobs:
            names.append(blob.name)
        return names

    def download_blob(self, bucket_name, source_blob_name, destination_file_name, source_blob_prefix=''):
        """Downloads a blob from the bucket."""
        bucket = self.storage_client.bucket(bucket_name)
        blobs = bucket.list_blobs()
        for blob in blobs:
            if blob.name.startswith(source_blob_prefix):
                print("downloading")
                blob.download_to_filename(destination_file_name)


def upload_gcp_wheel(gcp_storage, version, bucket_name, blob_path):
    """
    Uploads lightning wheel to it's designated GCP bucket.
    """
    whl_exists = False
    for file in os.listdir('dist'):
        if '.whl' in file:
            wheel = file
            whl_exists = True

    if whl_exists:
        lastest_path = Path(blob_path) / "latest" / wheel
        version_path = Path(blob_path) / version / wheel

        # delete existing object in "latest" directory
        gcp_storage.delete_blobs(bucket_name=bucket_name, prefix=f'{blob_path}/latest/')

        gcp_storage.upload_blob(
            bucket_name=bucket_name, source_file_name=f'dist/{wheel}', destination_blob_name=lastest_path
        )
        gcp_storage.upload_blob(
            bucket_name=bucket_name, source_file_name=f'dist/{wheel}', destination_blob_name=version_path
        )
    else:
        raise ValueError("You did not build a wheel for this project.")


class UploadCommandPyPi(Command):
    """Support setup.py upload-pypi."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        # skipcq: BAN-B605
        os.system('{0} setup.py sdist bdist_wheel'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        # skipcq: BAN-B605
        os.system('twine upload dist/*')

        sys.exit()


class UploadCommandGCP(Command):
    """Support setup.py upload-gcp."""

    description = 'Build and publish the package to GCP.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Checking environment variables…')
        check_environment_variables(['GOOGLE_APPLICATION_CREDENTIALS', 'GCP_GRID_BUCKET', 'GCP_BLOB_PATH'])

        self.status('Building Source and Wheel (universal) distribution…')
        # skipcq: BAN-B605
        os.system('{0} setup.py sdist bdist_wheel'.format(sys.executable))

        self.status('Uploading to GCP…')
        gcp_storage = GCPStorage()
        upload_gcp_wheel(
            gcp_storage=gcp_storage,
            version=VERSION,
            bucket_name=os.getenv('GCP_GRID_BUCKET'),
            blob_path=os.getenv('GCP_BLOB_PATH')
        )

        sys.exit()


class Install(install):
    """
    Override setup tools `install` command to do any pre/post install logic.
    """
    def run(self):
        def _post_install():
            def find_module_path():
                for p in sys.path:
                    if os.path.isdir(p) and "grid" in os.listdir(p):
                        return os.path.join(p, "grid")

            install_path = find_module_path()

            from grid.cli.utilities import install_autocomplete
            install_autocomplete()

        atexit.register(_post_install)
        install.run(self)


#  Where the magic happens:
setup(
    name=PACKAGE_NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=('tests', 'tests.*')),
    entry_points={
        'console_scripts': ['grid=grid.cli.__main__:main'],
    },
    long_description="Grid AI Command Line Interface",
    long_description_content_type="text/x-rst",
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    #  $ setup.py publish support.
    cmdclass={
        'pypi': UploadCommandPyPi,
        'gcp': UploadCommandGCP,
        'install': Install
    },
)



from grid.openapi import V1DeleteExperimentResponse, V1ListExperimentArtifactsResponse
from grid.openapi.models import (Externalv1Experiment, Body7, V1ExperimentState)

from grid.sdk.rest import GridRestClient
from grid.sdk.rest.exceptions import throw_with_message


@throw_with_message
def get_experiment_from_id(client: GridRestClient, cluster_id: str, experiment_id: str) -> Externalv1Experiment:
    return client.experiment_service_get_experiment(cluster_id=cluster_id, id=experiment_id)


@throw_with_message
def get_experiment_by_name(client: GridRestClient, cluster_id: str, experiment_name: str) -> Externalv1Experiment:
    exps = client.experiment_service_list_experiments(cluster_id=cluster_id, names=[experiment_name])
    if len(exps.experiments) == 0:
        raise KeyError(f"experiment with name {experiment_name} not found")
    elif len(exps.experiments) > 1:
        raise ValueError(f"multiple experiments found with the name {experiment_name}. please report error.")
    return exps.experiments[0]


@throw_with_message
def update_experiment(
    client: GridRestClient, cluster_id: str, experiment_id: str, desired_state: V1ExperimentState
) -> Externalv1Experiment:
    exp = get_experiment_from_id(client=client, cluster_id=cluster_id, experiment_id=experiment_id)
    spec = exp.spec
    spec.desired_state = desired_state
    update_exp_body = Body7(name=exp.name, spec=spec)
    return client.experiment_service_update_experiment(
        spec_cluster_id=cluster_id, id=experiment_id, body=update_exp_body
    )


@throw_with_message
def delete_experiment(client: GridRestClient, cluster_id: str, experiment_id: str) -> V1DeleteExperimentResponse:
    return client.experiment_service_delete_experiment(cluster_id=cluster_id, id=experiment_id)


@throw_with_message
def list_artifacts(
    client: GridRestClient,
    cluster_id: str,
    experiment_id: str,
    page_token: str = "",
    page_size: str = ""
) -> V1ListExperimentArtifactsResponse:
    return client.experiment_service_list_experiment_artifacts(
        cluster_id=cluster_id,
        id=experiment_id,
        page_token=page_token,
        page_size=page_size,
    )



from contextlib import contextmanager
import os
from pathlib import Path
from typing import Any


@contextmanager
def current_working_directory(path: Path):
    """Sets the cwd within the context

    Args:
        path (Path): The path to the cwd

    Yields:
        None
    """

    origin = Path().absolute()
    try:
        os.chdir(path)
        yield
    finally:
        os.chdir(origin)


def fill_object_from_response(obj: Any, response: Any) -> Any:
    """Fill the given object with the REST response using the object
    method `_setup_from_response`
    """
    obj._setup_from_response(response)  # noqa
    return obj



from contextlib import contextmanager
import os
from pathlib import Path
from shutil import copy2, rmtree
from typing import Optional, Set

import click

from grid.sdk.source_code.git import (
    check_github_repository,
    check_if_remote_head_is_different,
    check_if_uncommited_files,
    execute_git_command,
)
import grid.sdk.source_code.gridignore as gridignore
from grid.sdk.source_code.hashing import dirhash
from grid.sdk.utils import current_working_directory
from grid.sdk.utils.tar import tar_path
from grid.sdk.utils.uploader import S3Uploader


class LocalSourceCodeDir:
    """Utilities for managing local source-code used for creating Runs."""
    cache_location: Path = Path.home() / ".grid" / "cache" / "repositories"

    def __init__(self, path: Path):
        self.path = path

        # cache checksum version
        self._version: Optional[str] = None
        self._files_used_in_checksum: Optional[Set[Path]] = set()

        # create global cache location if it doesn't exist
        if not self.cache_location.exists():
            self.cache_location.mkdir(parents=True, exist_ok=True)

        # clean old cache entries
        self._prune_cache()

    @property
    def _ignore_patterns(self):
        """Generates set of paths from a `.gridignore` file. This is a wrapper for `gridignore.generate()`."""
        return gridignore.generate(src=self.path)

    @property
    def version(self):
        """
        Calculates the checksum of a local path.

        Parameters
        ----------
        path: Path
            Reference to a path.
        """
        # cache value to prevent doing this over again
        if self._version is not None:
            return self._version

        # stores both version and a set with the files used to generate the checksum
        self._files_used_in_checksum, self._version = dirhash(
            path=self.path, algorithm="blake2", ignore=self._ignore_patterns
        )
        return self._version

    @property
    def package_path(self):
        """Location to tarball in local cache."""
        filename = f"{self.version}.tar.gz"
        return self.cache_location / filename

    @contextmanager
    def packaging_session(self) -> Path:
        """Creates a local directory with source code that is used for creating a local source-code package."""
        session_path = self.cache_location / "packaging_sessions" / self.version
        try:
            rmtree(session_path, ignore_errors=True)

            # For consistency, copy all and only files included in checksum
            # Using two different ignore-mechanism is not wise, and actually cased problems here.
            for filename in self._files_used_in_checksum:
                new_path = session_path / filename.relative_to(self.path)
                new_path.parent.mkdir(parents=True, exist_ok=True)
                copy2(filename, new_path)

            yield session_path
        finally:
            rmtree(session_path, ignore_errors=True)

    def _prune_cache(self) -> None:
        """Prunes cache; only keeps the 10 most recent items."""
        packages = sorted(self.cache_location.iterdir(), key=os.path.getmtime)
        for package in packages[10:]:
            if package.is_file():
                package.unlink(missing_ok=True)

    def package(self) -> Path:
        """Packages local path using tar."""
        if self.package_path.exists():
            return self.package_path
        # create a packaging session if not available
        with self.packaging_session() as session_path:
            tar_path(source_path=session_path, target_file=str(self.package_path), compression=True)
        return self.package_path

    def upload(self, url: str) -> None:
        """Uploads package to URL, usually pre-signed URL.

        Notes
        -----
        Since we do not use multipart uploads here, we cannot upload any
        packaged repository files which have a size > 2GB.

        """
        package_size = self.package_path.stat().st_size
        if package_size > 2e9:
            raise OSError(
                "cannot upload directory code whose total fize size is greater than 2GB (2e9 bytes)."
            ) from None
        if package_size > 5e7:
            click.echo(
                "Your local dir upload size exceeds 50MB. We recommend the use of a .gridignore file to exclude unnecessary files from the upload process, as well as using datastores for dataset upload. See documentation for more details."
            )

        uploader = S3Uploader(
            presigned_urls={1: url},
            already_uploaded_parts=[],
            source_file=str(self.package_path),
            name=self.package_path.name,
            total_size=package_size,
            split_size=package_size
        )
        uploader.upload()


class GitRepository:
    """Utilities for managing git repositories."""
    def __init__(self, path: Path):
        self.path = Path(path).absolute()
        self.commit_sha = ""
        self.repo_name = ""
        with current_working_directory(path):
            check_github_repository()
            check_if_uncommited_files()
            check_if_remote_head_is_different()
            self.commit_sha = execute_git_command(['rev-parse', 'HEAD'])
            #  Get repo name
            repo_name = execute_git_command(["config", "--get", "remote.origin.url"])
            repo_name = repo_name.replace('git@github.com:', 'github.com/')
            repo_name = repo_name.replace('https://', '')
            repo_name = repo_name.replace('http://', '')  # noqa
            self.repo_name = repo_name.replace('.git', '')

            self.repository_root = execute_git_command(['rev-parse', '--show-toplevel'], cwd=self.path)

    def url(self):
        # TODO - Does this work
        return f"https://{self.repo_name}.git#ref={self.commit_sha}"

    def relative_working_dir(self) -> Path:
        return self.path.relative_to(self.repository_root)



from dataclasses import dataclass
import urllib.parse

from websocket import create_connection, WebSocketBadStatusException
from requests.models import PreparedRequest


@dataclass
class WebSocketClient:
    base_url: str
    token: str

    def connect(self, subscription_path, query_params=None):
        url = urllib.parse.urljoin(self.base_url, subscription_path)
        req = PreparedRequest()
        query_params["token"] = self.token
        req.prepare_url(url, query_params)
        # change the scheme only after prepare_url
        ws_url = req.url.replace("http", "ws", 1)
        # TODO - connection exception messages are ignored by the websocket-client package. We must be able to push a
        # fix upstream - https://github.com/websocket-client/websocket-client/issues/758
        return create_connection(ws_url)



#!/usr/bin/env python
# Copyright 2020 Grid AI Inc.
"""Entrypoint for the Grid CLI."""
import sys

import click

from grid.cli import rich_click
import grid.cli.cli as cli
from grid.cli.exceptions import ResourceNotFound
from grid.cli.utilities import introspect_module
from grid.metadata import __logo__, __version__
import grid.sdk.env as env


@rich_click.main_group(cls=rich_click.deprecate_and_alias({"train": "run", "interactive": "session", "cancel": "stop"}))
@click.option(
    '--debug', type=bool, help='Used for logging additional information for debugging purposes.', is_flag=True
)
@click.option(
    '-o',
    '--output',
    'output',
    type=click.Choice(['console', 'json'], case_sensitive=False),
    default="console",
    help='Output format'
)
@click.pass_context
def main(ctx: click.Context, debug: bool, output: str) -> None:
    """Grid CLI"""
    if not ctx.obj:
        ctx.obj = {}
    if debug:
        env.logger.info('Starting gridrunner in DEGUB mode.')
    ctx.obj['output'] = output

    env.DEBUG = debug


@main.command()
def version() -> None:
    """
    Prints CLI version to stdout.
    """
    logo = click.style(__logo__, fg='bright_green')
    click.echo(logo)

    version = f"""
                              Grid CLI ({__version__})
                            https://docs.grid.ai
    """
    click.echo(version)


@main.command()
def docs() -> None:
    """
    Open the CLI docs.
    """
    click.launch('https://docs.grid.ai/products/global-cli-configs')


#  Adds all CLI commands. Commands are introspected
#  from the cli module.
for command in introspect_module(cli):
    command: click.Command
    main: click.Group
    main.add_command(command)

if __name__ == '__main__':
    try:
        main.main(prog_name="grid", standalone_mode=False)
    except click.ClickException as e:
        e.show()
        sys.exit(e.exit_code)
    except ResourceNotFound as e:
        print(e)
        sys.exit(2)
    finally:
        pass



# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""


import pprint
import re  # noqa: F401
from typing import TYPE_CHECKING

import six

from grid.openapi.configuration import Configuration

if TYPE_CHECKING:
    from datetime import datetime
    from grid.openapi.models import *

class Externalv1ExperimentStatus(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'cost': 'float',
        'deleted_timestamp': 'datetime',
        'finish_timestamp': 'datetime',
        'hourly_cost': 'float',
        'message': 'str',
        'phase': 'V1ExperimentState',
        'start_timestamp': 'datetime'
    }

    attribute_map = {
        'cost': 'cost',
        'deleted_timestamp': 'deletedTimestamp',
        'finish_timestamp': 'finishTimestamp',
        'hourly_cost': 'hourlyCost',
        'message': 'message',
        'phase': 'phase',
        'start_timestamp': 'startTimestamp'
    }

    def __init__(self, cost: 'float' = None, deleted_timestamp: 'datetime' = None, finish_timestamp: 'datetime' = None, hourly_cost: 'float' = None, message: 'str' = None, phase: 'V1ExperimentState' = None, start_timestamp: 'datetime' = None, _configuration=None):  # noqa: E501
        """Externalv1ExperimentStatus - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._cost = None
        self._deleted_timestamp = None
        self._finish_timestamp = None
        self._hourly_cost = None
        self._message = None
        self._phase = None
        self._start_timestamp = None
        self.discriminator = None

        if cost is not None:
            self.cost = cost
        if deleted_timestamp is not None:
            self.deleted_timestamp = deleted_timestamp
        if finish_timestamp is not None:
            self.finish_timestamp = finish_timestamp
        if hourly_cost is not None:
            self.hourly_cost = hourly_cost
        if message is not None:
            self.message = message
        if phase is not None:
            self.phase = phase
        if start_timestamp is not None:
            self.start_timestamp = start_timestamp

    @property
    def cost(self) -> 'float':
        """Gets the cost of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The cost of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: float
        """
        return self._cost

    @cost.setter
    def cost(self, cost: 'float'):
        """Sets the cost of this Externalv1ExperimentStatus.


        :param cost: The cost of this Externalv1ExperimentStatus.  # noqa: E501
        :type: float
        """

        self._cost = cost

    @property
    def deleted_timestamp(self) -> 'datetime':
        """Gets the deleted_timestamp of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The deleted_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._deleted_timestamp

    @deleted_timestamp.setter
    def deleted_timestamp(self, deleted_timestamp: 'datetime'):
        """Sets the deleted_timestamp of this Externalv1ExperimentStatus.


        :param deleted_timestamp: The deleted_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :type: datetime
        """

        self._deleted_timestamp = deleted_timestamp

    @property
    def finish_timestamp(self) -> 'datetime':
        """Gets the finish_timestamp of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The finish_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._finish_timestamp

    @finish_timestamp.setter
    def finish_timestamp(self, finish_timestamp: 'datetime'):
        """Sets the finish_timestamp of this Externalv1ExperimentStatus.


        :param finish_timestamp: The finish_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :type: datetime
        """

        self._finish_timestamp = finish_timestamp

    @property
    def hourly_cost(self) -> 'float':
        """Gets the hourly_cost of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The hourly_cost of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: float
        """
        return self._hourly_cost

    @hourly_cost.setter
    def hourly_cost(self, hourly_cost: 'float'):
        """Sets the hourly_cost of this Externalv1ExperimentStatus.


        :param hourly_cost: The hourly_cost of this Externalv1ExperimentStatus.  # noqa: E501
        :type: float
        """

        self._hourly_cost = hourly_cost

    @property
    def message(self) -> 'str':
        """Gets the message of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The message of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message: 'str'):
        """Sets the message of this Externalv1ExperimentStatus.


        :param message: The message of this Externalv1ExperimentStatus.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def phase(self) -> 'V1ExperimentState':
        """Gets the phase of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The phase of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: V1ExperimentState
        """
        return self._phase

    @phase.setter
    def phase(self, phase: 'V1ExperimentState'):
        """Sets the phase of this Externalv1ExperimentStatus.


        :param phase: The phase of this Externalv1ExperimentStatus.  # noqa: E501
        :type: V1ExperimentState
        """

        self._phase = phase

    @property
    def start_timestamp(self) -> 'datetime':
        """Gets the start_timestamp of this Externalv1ExperimentStatus.  # noqa: E501


        :return: The start_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._start_timestamp

    @start_timestamp.setter
    def start_timestamp(self, start_timestamp: 'datetime'):
        """Sets the start_timestamp of this Externalv1ExperimentStatus.


        :param start_timestamp: The start_timestamp of this Externalv1ExperimentStatus.  # noqa: E501
        :type: datetime
        """

        self._start_timestamp = start_timestamp

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Externalv1ExperimentStatus, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'Externalv1ExperimentStatus') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, Externalv1ExperimentStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other: 'Externalv1ExperimentStatus') -> bool:
        """Returns true if both objects are not equal"""
        if not isinstance(other, Externalv1ExperimentStatus):
            return True

        return self.to_dict() != other.to_dict()



import csv
from dataclasses import asdict
from enum import Enum
import io
import json
from typing import Any, Dict, List, Union

from rich.console import Console
from rich.table import Table
from rich.text import Text

from grid.cli.sdk_record import Record


class SerializationFormat(Enum):
    """Formats used to serialize Record objects."""
    JSON = "json"
    CSV = "csv"
    TABLE = "table"


class Serializer:
    """
    Record parser will serialize Grid API results into desired formats.
    This is used in the context of a TUI where we may want to output the format
    of data into different formats, such as JSON, CSV, etc.

    Parameters
    ----------
    records: List[Record]
        List of Record objects
    """
    def __init__(self, records: List[Record]):
        self.records = records
        self.header = self.dict_records[0].keys()

    @property
    def dict_records(self) -> List[Dict[str, Any]]:
        """Dict[str, Any] representation of Record"""
        return [asdict(record) for record in self.records]

    def to_table(self) -> Table:
        """Serializes Record object to rich Table"""
        table = Table(*self.header, show_header=True, header_style="bold green")
        for record in self.dict_records:
            values = [Text(v) for v in record.values()]
            table.add_row(*values)
        return table

    def to_json(self) -> str:
        """Serializes Record objects to JSON."""
        return json.dumps(self.dict_records)

    def to_csv(self) -> str:
        """Serializes Record objects to CSV."""
        output = io.StringIO()
        writer = csv.DictWriter(output, quoting=csv.QUOTE_NONNUMERIC, fieldnames=self.header)
        writer.writeheader()
        for record in self.dict_records:
            writer.writerow(record)

        return output.getvalue()

    def generate(self, format: SerializationFormat = SerializationFormat.TABLE) -> Union[str]:
        """
        Serializes output in desired format.

        Parameters
        ----------
        format: SerializationFormat
            Type of output format to generate output in.
        """
        if format == SerializationFormat.TABLE:
            pass
        elif format == SerializationFormat.CSV:
            return self.to_csv()
        elif format == SerializationFormat.JSON:
            return self.to_json()

    def print(self, format: SerializationFormat = SerializationFormat.TABLE) -> None:
        console = Console()
        data = self.generate()
        console.print(data, highlight=False)



from dataclasses import dataclass
from typing import List

from grid.openapi import Externalv1Cluster, V1InstanceType
from grid.sdk.client import create_swagger_client
from grid.sdk.rest import GridRestClient
from grid.sdk.rest.clusters import list_clusters as rest_list_clusters

__all__ = ["get_instance_types", "list_clusters"]


def get_instance_types(cluster_id: str) -> List[V1InstanceType]:
    """
    Get the instance types for the given cluster
    """
    api_client = GridRestClient(create_swagger_client())
    resp = api_client.cluster_service_list_cluster_instance_types(id=cluster_id)
    return resp.instance_types


@dataclass
class ListClusters:
    default_cluster: str
    clusters: List[Externalv1Cluster]


# TODO - move this to sdk/clusters.py
def list_clusters(is_global: bool = False) -> ListClusters:
    """
    List the clusters for the user

    Parameters
    ----------
    is_global:
        Only returns global clusters
    """
    c = GridRestClient(create_swagger_client())
    resp = rest_list_clusters(c, is_global=is_global)
    return ListClusters(default_cluster=resp.default_cluster, clusters=resp.clusters if resp.clusters else [])



import base64
import textwrap
import uuid
from base64 import urlsafe_b64encode
from typing import Dict, List, Optional

import click
from click import Context
from rich.console import Console
from rich.prompt import Prompt
from rich.table import Table

from grid.cli import rich_click
from grid.openapi import V1CredentialType, V1Credential
from grid.sdk import env
from grid.sdk.client import create_swagger_client
from grid.sdk.credentials import (
    get_cluster_account_id,
    get_cluster_controlplane_aws_account_id,
    AWSBucketAccessSpec,
    BucketAccessCredential,
)
from grid.sdk.rest import GridRestClient
from grid.sdk.rest.credentials import create_credential, list_credentials, delete_credential
from grid.sdk.user import get_user_team_members, user_from_logged_in_account


@rich_click.group(invoke_without_command=True)
@click.pass_context
def credential(ctx: Context) -> None:
    """Manage the credentials associated with your Grid account.

    You can use these credentials to provide access to data stored in private s3 buckets.

    You can find more information about using private s3 buckets with Grid [link=https://docs.grid.ai]here[/link]
    Please note that this feature is only available to BYOC users.
    """
    if ctx.invoked_subcommand is None:
        ctx.invoke(list)


@credential.command()
@click.pass_context
@click.option(
    '--type',
    'credential_type',
    type=click.Choice(['s3']),
    default='s3',
    show_default=True,
    help='The credential type to create.',
)
@click.option(
    '--cluster',
    'cluster_id',
    type=str,
    required=False,
    default=env.CONTEXT,
    show_default=True,
    help='The cluster id where the credential will be created.',
)
@click.option(
    '--s3-external-id',
    's3_external_id',
    type=str,
    default=None,
    show_default=False,
    hidden=True,  # should only be used during testing
    help='Must be paired with `--type s3 & --s3-role-arn`',
)
@click.option(
    '--s3-role-arn',
    's3_role_arn',
    type=str,
    default=None,
    show_default=False,
    hidden=True,  # should only be used during testing
    help='Must be paired with `--type s3 & --s3-external-id`',
)
def create(
    ctx,
    credential_type: str,
    cluster_id: str,
    s3_external_id: Optional[str] = None,
    s3_role_arn: Optional[str] = None
):
    """Create a credential associated with your Grid account.

    You can use this credential to mount a Datastore from a private s3 bucket.
    """
    if credential_type == 's3':
        if s3_role_arn is not None and s3_external_id is not None:
            click.echo("skipping create credential interactive mode...")
            aws_bucket_access_spec = AWSBucketAccessSpec(
                role_arn=s3_role_arn,
                external_id=s3_external_id,
            )
        else:
            aws_bucket_access_spec = ask_user_input_to_create_s3_credential(cluster_id)
        try:
            aws_bucket_access_spec.validate()
        except ValueError as e:
            Console(width=80).print(
                f"\n[bold red]ValueError[/bold red]: {e}. This credential has "
                f"not been saved. Please rerun the last command to try again"
            )
            return
        credential = BucketAccessCredential(aws=aws_bucket_access_spec)
        created_cred = create_credential(
            c=GridRestClient(create_swagger_client()),
            cluster_id=cluster_id,
            credential_type=V1CredentialType.BUCKET_ACCESS,
            credential_value=urlsafe_b64encode(bytes(credential.to_json(), 'utf-8')).decode('utf-8'),
        )

    success_message = (
        f"\n"
        "[bold][green]Success![/bold][/green]\n"
        "\n"
        f"Your credential has been created on cluster [dim]{created_cred.cluster_id}[/dim]. It "
        f"can be referred to in the future by its ID: [dim]{created_cred.id}[/dim]. \n"
        f"\n"
        f"This can also be found in the output of [i][dim]grid credential list --cluster {created_cred.cluster_id}[/i][/dim]"
    )
    Console(width=80, highlight=False).print(success_message)


@credential.command()
@click.pass_context
@click.option(
    '--cluster',
    'cluster_id',
    type=str,
    required=False,
    default=env.CONTEXT,
    show_default=True,
    help='The cluster id where the credential will be created.',
)
@click.option(
    "--type",
    "credential_type",
    type=click.Choice(['s3']),
    required=False,
    default=None,
    show_default=True,
    help="filter credentials to list by type."
)
def list(ctx, cluster_id: str, credential_type: str):
    """List all credentials associated with your Grid account.
    """
    team_members = get_user_team_members()
    if len(team_members) == 0:
        team_members = []

    if credential_type == 's3':
        credentials = list_credentials(
            c=GridRestClient(create_swagger_client()),
            cluster_id=cluster_id,
            user_ids=[u.user_id for u in team_members] if len(team_members) == 0 else None,
            credential_type_in=V1CredentialType.BUCKET_ACCESS,
        )
    else:
        credentials = list_credentials(
            c=GridRestClient(create_swagger_client()),
            cluster_id=cluster_id,
            user_ids=[u.user_id for u in team_members] if len(team_members) == 0 else None,
        )

    user_id_name_map: Dict[str, str] = {}
    for member in team_members:
        user_id_name_map[member.user_id] = member.username

    # add current user to members list so we can find the username from id when printing creds table
    user = user_from_logged_in_account()
    user_id_name_map[user.user_id] = user.username

    print_creds_table(creds=credentials, user_id_name_map=user_id_name_map)


@credential.command()
@click.pass_context
@rich_click.argument("id", type=str, required=True, nargs=1, help="The credential ID to delete.")
@click.option(
    '--cluster',
    'cluster_id',
    type=str,
    required=False,
    default=env.CONTEXT,
    show_default=True,
    help='The cluster id where the credential will be created.',
)
def delete(ctx, id: str, cluster_id: str):
    """Use this command to delete a credential you have previously created.

    Warning: Any resource (datastore, session, experiment) which uses this credential will
    become unusable after it has been deleted.

    Please note that only the Grid user who created a credential will be able to delete it.
    """
    delete_credential(
        c=GridRestClient(create_swagger_client()),
        cluster_id=cluster_id,
        credential_id=id,
    )
    Console().print(f"[bold][green]Success![/bold][/green] Credential [dim]{id}[/dim] has been deleted.")


def print_creds_table(creds: List[V1Credential], user_id_name_map: Dict[str, str]) -> Console:
    """Prints table with cloud credentials."""
    if not creds:
        message = """
            No credentials available. Add new credentials with:

                grid credential create ...

            Or learn more by executing:

                grid credential --help
            """
        raise click.ClickException(message)

    table = Table(show_header=True, header_style="bold green")
    table.add_column("ID", style='dim')
    table.add_column('Type', justify='center')
    table.add_column('Credential', justify='center', overflow='fold')
    table.add_column('Created At', justify='center', overflow='fold')
    table.add_column('Cluster', justify='center')
    table.add_column('Owner', justify='left')

    for row in creds:
        if row.type == V1CredentialType.BUCKET_ACCESS:
            cred_value = base64.urlsafe_b64decode(row.credential).decode()
            access_credential: BucketAccessCredential = BucketAccessCredential.from_json(cred_value)
            table.add_row(
                row.id,
                "s3",
                access_credential.aws.to_json(),
                row.creation_timestamp.strftime("%a, %d %b %Y %H:%M:%S %Z"),
                row.cluster_id,
                user_id_name_map.get(row.user_id, "<unknown owner>"),
            )
        else:
            raise NotImplementedError(f"listing {row.type} credential type is not supported yet")

    console = Console()
    console.print(table)

    #  We return the console for testing purposes.
    #  This isn't actually used anywhere else.
    return console


def ask_user_input_to_create_s3_credential(cluster_id: str) -> AWSBucketAccessSpec:
    cluster_aws_account = get_cluster_account_id(cluster_id)
    controlplane_aws_account = get_cluster_controlplane_aws_account_id(cluster_id)
    generated_external_id = uuid.uuid4().hex

    if cluster_aws_account != controlplane_aws_account:
        trust_policy_principle_msg = f'''["{cluster_aws_account}", "{controlplane_aws_account}"]'''
    else:
        trust_policy_principle_msg = f'''"{cluster_aws_account}"'''

    introduction_msg = (
        "Please refer the the [link=https://docs.grid.ai]the documentation[/link] "
        "for how to create an AWS role and permission policy."
    )
    trust_policy_before_msg = "The trust policy for the role should be:"
    trust_policy_json = textwrap.dedent(
        f'''\
            {{
                "Version": "2012-10-17",
                "Statement": [
                    {{
                        "Effect": "Allow",
                        "Action": "sts:AssumeRole",
                        "Principal": {{
                            "AWS": {trust_policy_principle_msg}
                        }},
                        "Condition": {{
                            "StringEquals": {{
                                "sts:ExternalId": "{generated_external_id}"
                            }}
                        }}
                    }}
                ]
            }}
    '''
    )

    permission_policy_before_msg = "The permission policy attached to the role should be:"
    permission_policy_json = textwrap.dedent(
        f'''\
            {{
                "Version": "2012-10-17",
                "Statement": [
                    {{
                        "Effect": "Allow",
                        "Action": [
                            "s3:GetBucketLocation",
                            "s3:GetObject",
                            "s3:ListBucket"
                        ],
                        "Resource" : [
                            "arn:aws:s3:::<replace-with-bucket-name>",
                            "arn:aws:s3:::<replace-with-bucket-name>/*"
                        ]
                    }}
                ]
            }}
    '''
    )
    permission_policy_after_msg = (
        "Please be sure to change the [bold][italic]<replace-with-bucket-name>[/bold][/italic] "
        "field with the bucket name you wish to grant access to. More information can be found "
        "[link=https://docs.grid.ai]on the docs[/link]."
    )
    end_msg = (
        "[bold]Please Note[/bold]: when creating the role name in the AWS console, the role name "
        "[italic]MUST[/italic] begin with the prefix: [bold]grid-s3-access-[/bold] any valid "
        "characters can follow the prefix."
    )

    c = Console(width=80)
    c.print(introduction_msg)
    c.print("")
    c.print(trust_policy_before_msg)
    c.print_json(trust_policy_json)
    c.print("")
    c.print(permission_policy_before_msg)
    c.print_json(permission_policy_json)
    c.print("")
    c.print(permission_policy_after_msg)
    c.print("")
    c.print(end_msg)
    c.print("")
    role_arn = Prompt.ask("[cyan]When complete, please enter the role ARN[/cyan]")

    return AWSBucketAccessSpec(role_arn=role_arn, external_id=generated_external_id)



# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""


import pprint
import re  # noqa: F401
from typing import TYPE_CHECKING

import six

from grid.openapi.configuration import Configuration

if TYPE_CHECKING:
    from datetime import datetime
    from grid.openapi.models import *

class V1AWSClusterDriverSpec(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'abort_incomplete_datastore_num_days': 'int',
        'az_skip_ids': 'list[str]',
        'bastion_ami_override': 'str',
        'bucket_name': 'str',
        'builder_ami_name_override': 'str',
        'builder_ami_override': 'str',
        'builder_node_instance_type': 'str',
        'cpu_ami_override': 'str',
        'datastore_transition_to_intelligent_tiering_num_days': 'int',
        'enable_datastore_transition_to_intelligent_tiering': 'bool',
        'enable_dns_firewall': 'bool',
        'enable_experiment_transition_to_intelligent_tiering': 'bool',
        'experiment_transition_to_intelligent_tiering_num_days': 'int',
        'external_id': 'str',
        'fsx_enabled': 'bool',
        'gpu_ami_override': 'str',
        'guardduty_integration': 'bool',
        'instance_types': 'list[V1InstanceSpec]',
        'internal_gc_ttl': 'str',
        'internal_test_cluster': 'bool',
        'kubernetes_version': 'str',
        'management_node_instance_type': 'str',
        'original_cluster_name': 'str',
        'region': 'str',
        'role_arn': 'str',
        'tensorboard_node_instance_type': 'str',
        'x_multi_az_count': 'int'
    }

    attribute_map = {
        'abort_incomplete_datastore_num_days': 'abortIncompleteDatastoreNumDays',
        'az_skip_ids': 'azSkipIds',
        'bastion_ami_override': 'bastionAmiOverride',
        'bucket_name': 'bucketName',
        'builder_ami_name_override': 'builderAmiNameOverride',
        'builder_ami_override': 'builderAmiOverride',
        'builder_node_instance_type': 'builderNodeInstanceType',
        'cpu_ami_override': 'cpuAmiOverride',
        'datastore_transition_to_intelligent_tiering_num_days': 'datastoreTransitionToIntelligentTieringNumDays',
        'enable_datastore_transition_to_intelligent_tiering': 'enableDatastoreTransitionToIntelligentTiering',
        'enable_dns_firewall': 'enableDnsFirewall',
        'enable_experiment_transition_to_intelligent_tiering': 'enableExperimentTransitionToIntelligentTiering',
        'experiment_transition_to_intelligent_tiering_num_days': 'experimentTransitionToIntelligentTieringNumDays',
        'external_id': 'externalId',
        'fsx_enabled': 'fsxEnabled',
        'gpu_ami_override': 'gpuAmiOverride',
        'guardduty_integration': 'guarddutyIntegration',
        'instance_types': 'instanceTypes',
        'internal_gc_ttl': 'internalGcTtl',
        'internal_test_cluster': 'internalTestCluster',
        'kubernetes_version': 'kubernetesVersion',
        'management_node_instance_type': 'managementNodeInstanceType',
        'original_cluster_name': 'originalClusterName',
        'region': 'region',
        'role_arn': 'roleArn',
        'tensorboard_node_instance_type': 'tensorboardNodeInstanceType',
        'x_multi_az_count': 'xMultiAzCount'
    }

    def __init__(self, abort_incomplete_datastore_num_days: 'int' = None, az_skip_ids: 'list[str]' = None, bastion_ami_override: 'str' = None, bucket_name: 'str' = None, builder_ami_name_override: 'str' = None, builder_ami_override: 'str' = None, builder_node_instance_type: 'str' = None, cpu_ami_override: 'str' = None, datastore_transition_to_intelligent_tiering_num_days: 'int' = None, enable_datastore_transition_to_intelligent_tiering: 'bool' = None, enable_dns_firewall: 'bool' = None, enable_experiment_transition_to_intelligent_tiering: 'bool' = None, experiment_transition_to_intelligent_tiering_num_days: 'int' = None, external_id: 'str' = None, fsx_enabled: 'bool' = None, gpu_ami_override: 'str' = None, guardduty_integration: 'bool' = None, instance_types: 'list[V1InstanceSpec]' = None, internal_gc_ttl: 'str' = None, internal_test_cluster: 'bool' = None, kubernetes_version: 'str' = None, management_node_instance_type: 'str' = None, original_cluster_name: 'str' = None, region: 'str' = None, role_arn: 'str' = None, tensorboard_node_instance_type: 'str' = None, x_multi_az_count: 'int' = None, _configuration=None):  # noqa: E501
        """V1AWSClusterDriverSpec - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._abort_incomplete_datastore_num_days = None
        self._az_skip_ids = None
        self._bastion_ami_override = None
        self._bucket_name = None
        self._builder_ami_name_override = None
        self._builder_ami_override = None
        self._builder_node_instance_type = None
        self._cpu_ami_override = None
        self._datastore_transition_to_intelligent_tiering_num_days = None
        self._enable_datastore_transition_to_intelligent_tiering = None
        self._enable_dns_firewall = None
        self._enable_experiment_transition_to_intelligent_tiering = None
        self._experiment_transition_to_intelligent_tiering_num_days = None
        self._external_id = None
        self._fsx_enabled = None
        self._gpu_ami_override = None
        self._guardduty_integration = None
        self._instance_types = None
        self._internal_gc_ttl = None
        self._internal_test_cluster = None
        self._kubernetes_version = None
        self._management_node_instance_type = None
        self._original_cluster_name = None
        self._region = None
        self._role_arn = None
        self._tensorboard_node_instance_type = None
        self._x_multi_az_count = None
        self.discriminator = None

        if abort_incomplete_datastore_num_days is not None:
            self.abort_incomplete_datastore_num_days = abort_incomplete_datastore_num_days
        if az_skip_ids is not None:
            self.az_skip_ids = az_skip_ids
        if bastion_ami_override is not None:
            self.bastion_ami_override = bastion_ami_override
        if bucket_name is not None:
            self.bucket_name = bucket_name
        if builder_ami_name_override is not None:
            self.builder_ami_name_override = builder_ami_name_override
        if builder_ami_override is not None:
            self.builder_ami_override = builder_ami_override
        if builder_node_instance_type is not None:
            self.builder_node_instance_type = builder_node_instance_type
        if cpu_ami_override is not None:
            self.cpu_ami_override = cpu_ami_override
        if datastore_transition_to_intelligent_tiering_num_days is not None:
            self.datastore_transition_to_intelligent_tiering_num_days = datastore_transition_to_intelligent_tiering_num_days
        if enable_datastore_transition_to_intelligent_tiering is not None:
            self.enable_datastore_transition_to_intelligent_tiering = enable_datastore_transition_to_intelligent_tiering
        if enable_dns_firewall is not None:
            self.enable_dns_firewall = enable_dns_firewall
        if enable_experiment_transition_to_intelligent_tiering is not None:
            self.enable_experiment_transition_to_intelligent_tiering = enable_experiment_transition_to_intelligent_tiering
        if experiment_transition_to_intelligent_tiering_num_days is not None:
            self.experiment_transition_to_intelligent_tiering_num_days = experiment_transition_to_intelligent_tiering_num_days
        if external_id is not None:
            self.external_id = external_id
        if fsx_enabled is not None:
            self.fsx_enabled = fsx_enabled
        if gpu_ami_override is not None:
            self.gpu_ami_override = gpu_ami_override
        if guardduty_integration is not None:
            self.guardduty_integration = guardduty_integration
        if instance_types is not None:
            self.instance_types = instance_types
        if internal_gc_ttl is not None:
            self.internal_gc_ttl = internal_gc_ttl
        if internal_test_cluster is not None:
            self.internal_test_cluster = internal_test_cluster
        if kubernetes_version is not None:
            self.kubernetes_version = kubernetes_version
        if management_node_instance_type is not None:
            self.management_node_instance_type = management_node_instance_type
        if original_cluster_name is not None:
            self.original_cluster_name = original_cluster_name
        if region is not None:
            self.region = region
        if role_arn is not None:
            self.role_arn = role_arn
        if tensorboard_node_instance_type is not None:
            self.tensorboard_node_instance_type = tensorboard_node_instance_type
        if x_multi_az_count is not None:
            self.x_multi_az_count = x_multi_az_count

    @property
    def abort_incomplete_datastore_num_days(self) -> 'int':
        """Gets the abort_incomplete_datastore_num_days of this V1AWSClusterDriverSpec.  # noqa: E501

        how many days after initiating a datastore upload should the datastore be marked as failed and all data deleted if the \"upload\" is not complete. The value for tf abort_incomplete_datastore_mpu_days is derived from this value (this value + 1). Must be value >= 1 or a default value will be silently updated in place.  # noqa: E501

        :return: The abort_incomplete_datastore_num_days of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: int
        """
        return self._abort_incomplete_datastore_num_days

    @abort_incomplete_datastore_num_days.setter
    def abort_incomplete_datastore_num_days(self, abort_incomplete_datastore_num_days: 'int'):
        """Sets the abort_incomplete_datastore_num_days of this V1AWSClusterDriverSpec.

        how many days after initiating a datastore upload should the datastore be marked as failed and all data deleted if the \"upload\" is not complete. The value for tf abort_incomplete_datastore_mpu_days is derived from this value (this value + 1). Must be value >= 1 or a default value will be silently updated in place.  # noqa: E501

        :param abort_incomplete_datastore_num_days: The abort_incomplete_datastore_num_days of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: int
        """

        self._abort_incomplete_datastore_num_days = abort_incomplete_datastore_num_days

    @property
    def az_skip_ids(self) -> 'list[str]':
        """Gets the az_skip_ids of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The az_skip_ids of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: list[str]
        """
        return self._az_skip_ids

    @az_skip_ids.setter
    def az_skip_ids(self, az_skip_ids: 'list[str]'):
        """Sets the az_skip_ids of this V1AWSClusterDriverSpec.


        :param az_skip_ids: The az_skip_ids of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: list[str]
        """

        self._az_skip_ids = az_skip_ids

    @property
    def bastion_ami_override(self) -> 'str':
        """Gets the bastion_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The bastion_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._bastion_ami_override

    @bastion_ami_override.setter
    def bastion_ami_override(self, bastion_ami_override: 'str'):
        """Sets the bastion_ami_override of this V1AWSClusterDriverSpec.


        :param bastion_ami_override: The bastion_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._bastion_ami_override = bastion_ami_override

    @property
    def bucket_name(self) -> 'str':
        """Gets the bucket_name of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The bucket_name of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._bucket_name

    @bucket_name.setter
    def bucket_name(self, bucket_name: 'str'):
        """Sets the bucket_name of this V1AWSClusterDriverSpec.


        :param bucket_name: The bucket_name of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._bucket_name = bucket_name

    @property
    def builder_ami_name_override(self) -> 'str':
        """Gets the builder_ami_name_override of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The builder_ami_name_override of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._builder_ami_name_override

    @builder_ami_name_override.setter
    def builder_ami_name_override(self, builder_ami_name_override: 'str'):
        """Sets the builder_ami_name_override of this V1AWSClusterDriverSpec.


        :param builder_ami_name_override: The builder_ami_name_override of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._builder_ami_name_override = builder_ami_name_override

    @property
    def builder_ami_override(self) -> 'str':
        """Gets the builder_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The builder_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._builder_ami_override

    @builder_ami_override.setter
    def builder_ami_override(self, builder_ami_override: 'str'):
        """Sets the builder_ami_override of this V1AWSClusterDriverSpec.


        :param builder_ami_override: The builder_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._builder_ami_override = builder_ami_override

    @property
    def builder_node_instance_type(self) -> 'str':
        """Gets the builder_node_instance_type of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The builder_node_instance_type of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._builder_node_instance_type

    @builder_node_instance_type.setter
    def builder_node_instance_type(self, builder_node_instance_type: 'str'):
        """Sets the builder_node_instance_type of this V1AWSClusterDriverSpec.


        :param builder_node_instance_type: The builder_node_instance_type of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._builder_node_instance_type = builder_node_instance_type

    @property
    def cpu_ami_override(self) -> 'str':
        """Gets the cpu_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The cpu_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._cpu_ami_override

    @cpu_ami_override.setter
    def cpu_ami_override(self, cpu_ami_override: 'str'):
        """Sets the cpu_ami_override of this V1AWSClusterDriverSpec.


        :param cpu_ami_override: The cpu_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._cpu_ami_override = cpu_ami_override

    @property
    def datastore_transition_to_intelligent_tiering_num_days(self) -> 'int':
        """Gets the datastore_transition_to_intelligent_tiering_num_days of this V1AWSClusterDriverSpec.  # noqa: E501

        how many days after datastore blob upload should the object transition to INTELLIGENT_TIERING storage class. If variable `enable_datastore_transition_to_intelligent_tiering` is `false` then this value is ignored.  # noqa: E501

        :return: The datastore_transition_to_intelligent_tiering_num_days of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: int
        """
        return self._datastore_transition_to_intelligent_tiering_num_days

    @datastore_transition_to_intelligent_tiering_num_days.setter
    def datastore_transition_to_intelligent_tiering_num_days(self, datastore_transition_to_intelligent_tiering_num_days: 'int'):
        """Sets the datastore_transition_to_intelligent_tiering_num_days of this V1AWSClusterDriverSpec.

        how many days after datastore blob upload should the object transition to INTELLIGENT_TIERING storage class. If variable `enable_datastore_transition_to_intelligent_tiering` is `false` then this value is ignored.  # noqa: E501

        :param datastore_transition_to_intelligent_tiering_num_days: The datastore_transition_to_intelligent_tiering_num_days of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: int
        """

        self._datastore_transition_to_intelligent_tiering_num_days = datastore_transition_to_intelligent_tiering_num_days

    @property
    def enable_datastore_transition_to_intelligent_tiering(self) -> 'bool':
        """Gets the enable_datastore_transition_to_intelligent_tiering of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The enable_datastore_transition_to_intelligent_tiering of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: bool
        """
        return self._enable_datastore_transition_to_intelligent_tiering

    @enable_datastore_transition_to_intelligent_tiering.setter
    def enable_datastore_transition_to_intelligent_tiering(self, enable_datastore_transition_to_intelligent_tiering: 'bool'):
        """Sets the enable_datastore_transition_to_intelligent_tiering of this V1AWSClusterDriverSpec.


        :param enable_datastore_transition_to_intelligent_tiering: The enable_datastore_transition_to_intelligent_tiering of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: bool
        """

        self._enable_datastore_transition_to_intelligent_tiering = enable_datastore_transition_to_intelligent_tiering

    @property
    def enable_dns_firewall(self) -> 'bool':
        """Gets the enable_dns_firewall of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The enable_dns_firewall of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: bool
        """
        return self._enable_dns_firewall

    @enable_dns_firewall.setter
    def enable_dns_firewall(self, enable_dns_firewall: 'bool'):
        """Sets the enable_dns_firewall of this V1AWSClusterDriverSpec.


        :param enable_dns_firewall: The enable_dns_firewall of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: bool
        """

        self._enable_dns_firewall = enable_dns_firewall

    @property
    def enable_experiment_transition_to_intelligent_tiering(self) -> 'bool':
        """Gets the enable_experiment_transition_to_intelligent_tiering of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The enable_experiment_transition_to_intelligent_tiering of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: bool
        """
        return self._enable_experiment_transition_to_intelligent_tiering

    @enable_experiment_transition_to_intelligent_tiering.setter
    def enable_experiment_transition_to_intelligent_tiering(self, enable_experiment_transition_to_intelligent_tiering: 'bool'):
        """Sets the enable_experiment_transition_to_intelligent_tiering of this V1AWSClusterDriverSpec.


        :param enable_experiment_transition_to_intelligent_tiering: The enable_experiment_transition_to_intelligent_tiering of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: bool
        """

        self._enable_experiment_transition_to_intelligent_tiering = enable_experiment_transition_to_intelligent_tiering

    @property
    def experiment_transition_to_intelligent_tiering_num_days(self) -> 'int':
        """Gets the experiment_transition_to_intelligent_tiering_num_days of this V1AWSClusterDriverSpec.  # noqa: E501

        how many days after datastore blob upload should the object transition to INTELLIGENT_TIERING storage class. If variable `enable_experiment_transition_to_intelligent_tiering` is `false` then this value is ignored.  # noqa: E501

        :return: The experiment_transition_to_intelligent_tiering_num_days of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: int
        """
        return self._experiment_transition_to_intelligent_tiering_num_days

    @experiment_transition_to_intelligent_tiering_num_days.setter
    def experiment_transition_to_intelligent_tiering_num_days(self, experiment_transition_to_intelligent_tiering_num_days: 'int'):
        """Sets the experiment_transition_to_intelligent_tiering_num_days of this V1AWSClusterDriverSpec.

        how many days after datastore blob upload should the object transition to INTELLIGENT_TIERING storage class. If variable `enable_experiment_transition_to_intelligent_tiering` is `false` then this value is ignored.  # noqa: E501

        :param experiment_transition_to_intelligent_tiering_num_days: The experiment_transition_to_intelligent_tiering_num_days of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: int
        """

        self._experiment_transition_to_intelligent_tiering_num_days = experiment_transition_to_intelligent_tiering_num_days

    @property
    def external_id(self) -> 'str':
        """Gets the external_id of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The external_id of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._external_id

    @external_id.setter
    def external_id(self, external_id: 'str'):
        """Sets the external_id of this V1AWSClusterDriverSpec.


        :param external_id: The external_id of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._external_id = external_id

    @property
    def fsx_enabled(self) -> 'bool':
        """Gets the fsx_enabled of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The fsx_enabled of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: bool
        """
        return self._fsx_enabled

    @fsx_enabled.setter
    def fsx_enabled(self, fsx_enabled: 'bool'):
        """Sets the fsx_enabled of this V1AWSClusterDriverSpec.


        :param fsx_enabled: The fsx_enabled of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: bool
        """

        self._fsx_enabled = fsx_enabled

    @property
    def gpu_ami_override(self) -> 'str':
        """Gets the gpu_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The gpu_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._gpu_ami_override

    @gpu_ami_override.setter
    def gpu_ami_override(self, gpu_ami_override: 'str'):
        """Sets the gpu_ami_override of this V1AWSClusterDriverSpec.


        :param gpu_ami_override: The gpu_ami_override of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._gpu_ami_override = gpu_ami_override

    @property
    def guardduty_integration(self) -> 'bool':
        """Gets the guardduty_integration of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The guardduty_integration of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: bool
        """
        return self._guardduty_integration

    @guardduty_integration.setter
    def guardduty_integration(self, guardduty_integration: 'bool'):
        """Sets the guardduty_integration of this V1AWSClusterDriverSpec.


        :param guardduty_integration: The guardduty_integration of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: bool
        """

        self._guardduty_integration = guardduty_integration

    @property
    def instance_types(self) -> 'list[V1InstanceSpec]':
        """Gets the instance_types of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The instance_types of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: list[V1InstanceSpec]
        """
        return self._instance_types

    @instance_types.setter
    def instance_types(self, instance_types: 'list[V1InstanceSpec]'):
        """Sets the instance_types of this V1AWSClusterDriverSpec.


        :param instance_types: The instance_types of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: list[V1InstanceSpec]
        """

        self._instance_types = instance_types

    @property
    def internal_gc_ttl(self) -> 'str':
        """Gets the internal_gc_ttl of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The internal_gc_ttl of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._internal_gc_ttl

    @internal_gc_ttl.setter
    def internal_gc_ttl(self, internal_gc_ttl: 'str'):
        """Sets the internal_gc_ttl of this V1AWSClusterDriverSpec.


        :param internal_gc_ttl: The internal_gc_ttl of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._internal_gc_ttl = internal_gc_ttl

    @property
    def internal_test_cluster(self) -> 'bool':
        """Gets the internal_test_cluster of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The internal_test_cluster of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: bool
        """
        return self._internal_test_cluster

    @internal_test_cluster.setter
    def internal_test_cluster(self, internal_test_cluster: 'bool'):
        """Sets the internal_test_cluster of this V1AWSClusterDriverSpec.


        :param internal_test_cluster: The internal_test_cluster of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: bool
        """

        self._internal_test_cluster = internal_test_cluster

    @property
    def kubernetes_version(self) -> 'str':
        """Gets the kubernetes_version of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The kubernetes_version of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._kubernetes_version

    @kubernetes_version.setter
    def kubernetes_version(self, kubernetes_version: 'str'):
        """Sets the kubernetes_version of this V1AWSClusterDriverSpec.


        :param kubernetes_version: The kubernetes_version of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._kubernetes_version = kubernetes_version

    @property
    def management_node_instance_type(self) -> 'str':
        """Gets the management_node_instance_type of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The management_node_instance_type of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._management_node_instance_type

    @management_node_instance_type.setter
    def management_node_instance_type(self, management_node_instance_type: 'str'):
        """Sets the management_node_instance_type of this V1AWSClusterDriverSpec.


        :param management_node_instance_type: The management_node_instance_type of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._management_node_instance_type = management_node_instance_type

    @property
    def original_cluster_name(self) -> 'str':
        """Gets the original_cluster_name of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The original_cluster_name of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._original_cluster_name

    @original_cluster_name.setter
    def original_cluster_name(self, original_cluster_name: 'str'):
        """Sets the original_cluster_name of this V1AWSClusterDriverSpec.


        :param original_cluster_name: The original_cluster_name of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._original_cluster_name = original_cluster_name

    @property
    def region(self) -> 'str':
        """Gets the region of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The region of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._region

    @region.setter
    def region(self, region: 'str'):
        """Sets the region of this V1AWSClusterDriverSpec.


        :param region: The region of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._region = region

    @property
    def role_arn(self) -> 'str':
        """Gets the role_arn of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The role_arn of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._role_arn

    @role_arn.setter
    def role_arn(self, role_arn: 'str'):
        """Sets the role_arn of this V1AWSClusterDriverSpec.


        :param role_arn: The role_arn of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._role_arn = role_arn

    @property
    def tensorboard_node_instance_type(self) -> 'str':
        """Gets the tensorboard_node_instance_type of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The tensorboard_node_instance_type of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: str
        """
        return self._tensorboard_node_instance_type

    @tensorboard_node_instance_type.setter
    def tensorboard_node_instance_type(self, tensorboard_node_instance_type: 'str'):
        """Sets the tensorboard_node_instance_type of this V1AWSClusterDriverSpec.


        :param tensorboard_node_instance_type: The tensorboard_node_instance_type of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: str
        """

        self._tensorboard_node_instance_type = tensorboard_node_instance_type

    @property
    def x_multi_az_count(self) -> 'int':
        """Gets the x_multi_az_count of this V1AWSClusterDriverSpec.  # noqa: E501


        :return: The x_multi_az_count of this V1AWSClusterDriverSpec.  # noqa: E501
        :rtype: int
        """
        return self._x_multi_az_count

    @x_multi_az_count.setter
    def x_multi_az_count(self, x_multi_az_count: 'int'):
        """Sets the x_multi_az_count of this V1AWSClusterDriverSpec.


        :param x_multi_az_count: The x_multi_az_count of this V1AWSClusterDriverSpec.  # noqa: E501
        :type: int
        """

        self._x_multi_az_count = x_multi_az_count

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1AWSClusterDriverSpec, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'V1AWSClusterDriverSpec') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, V1AWSClusterDriverSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other: 'V1AWSClusterDriverSpec') -> bool:
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1AWSClusterDriverSpec):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""


from __future__ import absolute_import

import re  # noqa: F401
from typing import TYPE_CHECKING, Any

# python 2 and python 3 compatibility library
import six

from grid.openapi.api_client import ApiClient

if TYPE_CHECKING:
    from datetime import datetime
    from grid.openapi.models import *


class ExperimentServiceApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def experiment_service_delete_experiment(self, cluster_id: 'str', id: 'str', **kwargs) -> 'V1DeleteExperimentResponse':  # noqa: E501
        """experiment_service_delete_experiment  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_delete_experiment(cluster_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster_id: (required)
        :param str id: (required)
        :return: V1DeleteExperimentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.experiment_service_delete_experiment_with_http_info(cluster_id, id, **kwargs)  # noqa: E501
        else:
            (data) = self.experiment_service_delete_experiment_with_http_info(cluster_id, id, **kwargs)  # noqa: E501
            return data

    def experiment_service_delete_experiment_with_http_info(self, cluster_id: 'str', id: 'str', **kwargs) -> 'V1DeleteExperimentResponse':  # noqa: E501
        """experiment_service_delete_experiment  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_delete_experiment_with_http_info(cluster_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster_id: (required)
        :param str id: (required)
        :return: V1DeleteExperimentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['cluster_id', 'id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method experiment_service_delete_experiment" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'cluster_id' is set
        if self.api_client.client_side_validation and ('cluster_id' not in params or
                                                       params['cluster_id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `cluster_id` when calling `experiment_service_delete_experiment`")  # noqa: E501
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in params or
                                                       params['id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `id` when calling `experiment_service_delete_experiment`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'cluster_id' in params:
            path_params['clusterId'] = params['cluster_id']  # noqa: E501
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/core/{clusterId}/experiments/{id}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1DeleteExperimentResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def experiment_service_get_experiment(self, cluster_id: 'str', id: 'str', **kwargs) -> 'Externalv1Experiment':  # noqa: E501
        """experiment_service_get_experiment  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_get_experiment(cluster_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster_id: (required)
        :param str id: (required)
        :return: Externalv1Experiment
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.experiment_service_get_experiment_with_http_info(cluster_id, id, **kwargs)  # noqa: E501
        else:
            (data) = self.experiment_service_get_experiment_with_http_info(cluster_id, id, **kwargs)  # noqa: E501
            return data

    def experiment_service_get_experiment_with_http_info(self, cluster_id: 'str', id: 'str', **kwargs) -> 'Externalv1Experiment':  # noqa: E501
        """experiment_service_get_experiment  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_get_experiment_with_http_info(cluster_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster_id: (required)
        :param str id: (required)
        :return: Externalv1Experiment
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['cluster_id', 'id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method experiment_service_get_experiment" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'cluster_id' is set
        if self.api_client.client_side_validation and ('cluster_id' not in params or
                                                       params['cluster_id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `cluster_id` when calling `experiment_service_get_experiment`")  # noqa: E501
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in params or
                                                       params['id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `id` when calling `experiment_service_get_experiment`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'cluster_id' in params:
            path_params['clusterId'] = params['cluster_id']  # noqa: E501
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/core/{clusterId}/experiments/{id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Externalv1Experiment',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def experiment_service_list_experiment_artifacts(self, cluster_id: 'str', id: 'str', **kwargs) -> 'V1ListExperimentArtifactsResponse':  # noqa: E501
        """experiment_service_list_experiment_artifacts  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_list_experiment_artifacts(cluster_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster_id: (required)
        :param str id: (required)
        :param str page_token:
        :param str page_size:
        :return: V1ListExperimentArtifactsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.experiment_service_list_experiment_artifacts_with_http_info(cluster_id, id, **kwargs)  # noqa: E501
        else:
            (data) = self.experiment_service_list_experiment_artifacts_with_http_info(cluster_id, id, **kwargs)  # noqa: E501
            return data

    def experiment_service_list_experiment_artifacts_with_http_info(self, cluster_id: 'str', id: 'str', **kwargs) -> 'V1ListExperimentArtifactsResponse':  # noqa: E501
        """experiment_service_list_experiment_artifacts  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_list_experiment_artifacts_with_http_info(cluster_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster_id: (required)
        :param str id: (required)
        :param str page_token:
        :param str page_size:
        :return: V1ListExperimentArtifactsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['cluster_id', 'id', 'page_token', 'page_size']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method experiment_service_list_experiment_artifacts" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'cluster_id' is set
        if self.api_client.client_side_validation and ('cluster_id' not in params or
                                                       params['cluster_id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `cluster_id` when calling `experiment_service_list_experiment_artifacts`")  # noqa: E501
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in params or
                                                       params['id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `id` when calling `experiment_service_list_experiment_artifacts`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'cluster_id' in params:
            path_params['clusterId'] = params['cluster_id']  # noqa: E501
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []
        if 'page_token' in params:
            query_params.append(('pageToken', params['page_token']))  # noqa: E501
        if 'page_size' in params:
            query_params.append(('pageSize', params['page_size']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/core/{clusterId}/experiments/{id}/artifacts', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1ListExperimentArtifactsResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def experiment_service_list_experiment_events(self, cluster_id: 'str', id: 'str', **kwargs) -> 'V1ListExperimentEventsResponse':  # noqa: E501
        """experiment_service_list_experiment_events  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_list_experiment_events(cluster_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster_id: (required)
        :param str id: (required)
        :return: V1ListExperimentEventsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.experiment_service_list_experiment_events_with_http_info(cluster_id, id, **kwargs)  # noqa: E501
        else:
            (data) = self.experiment_service_list_experiment_events_with_http_info(cluster_id, id, **kwargs)  # noqa: E501
            return data

    def experiment_service_list_experiment_events_with_http_info(self, cluster_id: 'str', id: 'str', **kwargs) -> 'V1ListExperimentEventsResponse':  # noqa: E501
        """experiment_service_list_experiment_events  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_list_experiment_events_with_http_info(cluster_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster_id: (required)
        :param str id: (required)
        :return: V1ListExperimentEventsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['cluster_id', 'id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method experiment_service_list_experiment_events" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'cluster_id' is set
        if self.api_client.client_side_validation and ('cluster_id' not in params or
                                                       params['cluster_id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `cluster_id` when calling `experiment_service_list_experiment_events`")  # noqa: E501
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in params or
                                                       params['id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `id` when calling `experiment_service_list_experiment_events`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'cluster_id' in params:
            path_params['clusterId'] = params['cluster_id']  # noqa: E501
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/core/{clusterId}/experiments/{id}/events', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1ListExperimentEventsResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def experiment_service_list_experiments(self, cluster_id: 'str', **kwargs) -> 'V1ListExperimentsResponse':  # noqa: E501
        """experiment_service_list_experiments  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_list_experiments(cluster_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster_id: (required)
        :param str page_token:
        :param str limit:
        :param list[str] names:
        :param list[str] phase_in:  - EXPERIMENT_STATE_PENDING: experiment has been created on the cluster but hasn't yet started running after image is built and pushed, status gets updated to pending (the experiment pod has been created on the cluster and we are waiting for it to start running)  - EXPERIMENT_STATE_QUEUED: the image is being actively being built legacy code call this queued This state is not actively used in the code It's set by the python run.py on start  - EXPERIMENT_STATE_RUNNING: begins image building
        :param list[str] phase_not_in:  - EXPERIMENT_STATE_PENDING: experiment has been created on the cluster but hasn't yet started running after image is built and pushed, status gets updated to pending (the experiment pod has been created on the cluster and we are waiting for it to start running)  - EXPERIMENT_STATE_QUEUED: the image is being actively being built legacy code call this queued This state is not actively used in the code It's set by the python run.py on start  - EXPERIMENT_STATE_RUNNING: begins image building
        :param list[str] run_ids:
        :return: V1ListExperimentsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.experiment_service_list_experiments_with_http_info(cluster_id, **kwargs)  # noqa: E501
        else:
            (data) = self.experiment_service_list_experiments_with_http_info(cluster_id, **kwargs)  # noqa: E501
            return data

    def experiment_service_list_experiments_with_http_info(self, cluster_id: 'str', **kwargs) -> 'V1ListExperimentsResponse':  # noqa: E501
        """experiment_service_list_experiments  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_list_experiments_with_http_info(cluster_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster_id: (required)
        :param str page_token:
        :param str limit:
        :param list[str] names:
        :param list[str] phase_in:  - EXPERIMENT_STATE_PENDING: experiment has been created on the cluster but hasn't yet started running after image is built and pushed, status gets updated to pending (the experiment pod has been created on the cluster and we are waiting for it to start running)  - EXPERIMENT_STATE_QUEUED: the image is being actively being built legacy code call this queued This state is not actively used in the code It's set by the python run.py on start  - EXPERIMENT_STATE_RUNNING: begins image building
        :param list[str] phase_not_in:  - EXPERIMENT_STATE_PENDING: experiment has been created on the cluster but hasn't yet started running after image is built and pushed, status gets updated to pending (the experiment pod has been created on the cluster and we are waiting for it to start running)  - EXPERIMENT_STATE_QUEUED: the image is being actively being built legacy code call this queued This state is not actively used in the code It's set by the python run.py on start  - EXPERIMENT_STATE_RUNNING: begins image building
        :param list[str] run_ids:
        :return: V1ListExperimentsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['cluster_id', 'page_token', 'limit', 'names', 'phase_in', 'phase_not_in', 'run_ids']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method experiment_service_list_experiments" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'cluster_id' is set
        if self.api_client.client_side_validation and ('cluster_id' not in params or
                                                       params['cluster_id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `cluster_id` when calling `experiment_service_list_experiments`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'cluster_id' in params:
            path_params['clusterId'] = params['cluster_id']  # noqa: E501

        query_params = []
        if 'page_token' in params:
            query_params.append(('pageToken', params['page_token']))  # noqa: E501
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'names' in params:
            query_params.append(('names', params['names']))  # noqa: E501
            collection_formats['names'] = 'multi'  # noqa: E501
        if 'phase_in' in params:
            query_params.append(('phaseIn', params['phase_in']))  # noqa: E501
            collection_formats['phaseIn'] = 'multi'  # noqa: E501
        if 'phase_not_in' in params:
            query_params.append(('phaseNotIn', params['phase_not_in']))  # noqa: E501
            collection_formats['phaseNotIn'] = 'multi'  # noqa: E501
        if 'run_ids' in params:
            query_params.append(('runIds', params['run_ids']))  # noqa: E501
            collection_formats['runIds'] = 'multi'  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/core/{clusterId}/experiments', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1ListExperimentsResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def experiment_service_update_experiment(self, spec_cluster_id: 'str', id: 'str', body: 'Body7', **kwargs) -> 'Externalv1Experiment':  # noqa: E501
        """experiment_service_update_experiment  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_update_experiment(spec_cluster_id, id, body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str spec_cluster_id: (required)
        :param str id: (required)
        :param Body7 body: (required)
        :return: Externalv1Experiment
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.experiment_service_update_experiment_with_http_info(spec_cluster_id, id, body, **kwargs)  # noqa: E501
        else:
            (data) = self.experiment_service_update_experiment_with_http_info(spec_cluster_id, id, body, **kwargs)  # noqa: E501
            return data

    def experiment_service_update_experiment_with_http_info(self, spec_cluster_id: 'str', id: 'str', body: 'Body7', **kwargs) -> 'Externalv1Experiment':  # noqa: E501
        """experiment_service_update_experiment  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_service_update_experiment_with_http_info(spec_cluster_id, id, body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str spec_cluster_id: (required)
        :param str id: (required)
        :param Body7 body: (required)
        :return: Externalv1Experiment
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['spec_cluster_id', 'id', 'body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method experiment_service_update_experiment" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'spec_cluster_id' is set
        if self.api_client.client_side_validation and ('spec_cluster_id' not in params or
                                                       params['spec_cluster_id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `spec_cluster_id` when calling `experiment_service_update_experiment`")  # noqa: E501
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in params or
                                                       params['id'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `id` when calling `experiment_service_update_experiment`")  # noqa: E501
        # verify the required parameter 'body' is set
        if self.api_client.client_side_validation and ('body' not in params or
                                                       params['body'] is None):  # noqa: E501
            raise ValueError("Missing the required parameter `body` when calling `experiment_service_update_experiment`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'spec_cluster_id' in params:
            path_params['spec.clusterId'] = params['spec_cluster_id']  # noqa: E501
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/core/{spec.clusterId}/experiments/{id}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Externalv1Experiment',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""


import pprint
import re  # noqa: F401
from typing import TYPE_CHECKING

import six

from grid.openapi.configuration import Configuration

if TYPE_CHECKING:
    from datetime import datetime
    from grid.openapi.models import *

class V1Credential(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'cluster_id': 'str',
        'creation_timestamp': 'datetime',
        'credential': 'str',
        'deletion_timestamp': 'datetime',
        'id': 'str',
        'type': 'V1CredentialType',
        'update_timestamp': 'datetime',
        'user_id': 'str'
    }

    attribute_map = {
        'cluster_id': 'clusterId',
        'creation_timestamp': 'creationTimestamp',
        'credential': 'credential',
        'deletion_timestamp': 'deletionTimestamp',
        'id': 'id',
        'type': 'type',
        'update_timestamp': 'updateTimestamp',
        'user_id': 'userId'
    }

    def __init__(self, cluster_id: 'str' = None, creation_timestamp: 'datetime' = None, credential: 'str' = None, deletion_timestamp: 'datetime' = None, id: 'str' = None, type: 'V1CredentialType' = None, update_timestamp: 'datetime' = None, user_id: 'str' = None, _configuration=None):  # noqa: E501
        """V1Credential - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._cluster_id = None
        self._creation_timestamp = None
        self._credential = None
        self._deletion_timestamp = None
        self._id = None
        self._type = None
        self._update_timestamp = None
        self._user_id = None
        self.discriminator = None

        if cluster_id is not None:
            self.cluster_id = cluster_id
        if creation_timestamp is not None:
            self.creation_timestamp = creation_timestamp
        if credential is not None:
            self.credential = credential
        if deletion_timestamp is not None:
            self.deletion_timestamp = deletion_timestamp
        if id is not None:
            self.id = id
        if type is not None:
            self.type = type
        if update_timestamp is not None:
            self.update_timestamp = update_timestamp
        if user_id is not None:
            self.user_id = user_id

    @property
    def cluster_id(self) -> 'str':
        """Gets the cluster_id of this V1Credential.  # noqa: E501


        :return: The cluster_id of this V1Credential.  # noqa: E501
        :rtype: str
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id: 'str'):
        """Sets the cluster_id of this V1Credential.


        :param cluster_id: The cluster_id of this V1Credential.  # noqa: E501
        :type: str
        """

        self._cluster_id = cluster_id

    @property
    def creation_timestamp(self) -> 'datetime':
        """Gets the creation_timestamp of this V1Credential.  # noqa: E501


        :return: The creation_timestamp of this V1Credential.  # noqa: E501
        :rtype: datetime
        """
        return self._creation_timestamp

    @creation_timestamp.setter
    def creation_timestamp(self, creation_timestamp: 'datetime'):
        """Sets the creation_timestamp of this V1Credential.


        :param creation_timestamp: The creation_timestamp of this V1Credential.  # noqa: E501
        :type: datetime
        """

        self._creation_timestamp = creation_timestamp

    @property
    def credential(self) -> 'str':
        """Gets the credential of this V1Credential.  # noqa: E501

        credential is a serialized json blob of a particular credential type. this is done since the protobuf oneof keyword presents a pretty horrible UX. it is upon the implementer of each `CredentialType` to implement their own serialization / deserialization routines.  # noqa: E501

        :return: The credential of this V1Credential.  # noqa: E501
        :rtype: str
        """
        return self._credential

    @credential.setter
    def credential(self, credential: 'str'):
        """Sets the credential of this V1Credential.

        credential is a serialized json blob of a particular credential type. this is done since the protobuf oneof keyword presents a pretty horrible UX. it is upon the implementer of each `CredentialType` to implement their own serialization / deserialization routines.  # noqa: E501

        :param credential: The credential of this V1Credential.  # noqa: E501
        :type: str
        """
        if (self._configuration.client_side_validation and
                credential is not None and not re.search(r'^(?:[A-Za-z0-9+\/]{4})*(?:[A-Za-z0-9+\/]{2}==|[A-Za-z0-9+\/]{3}=)?$', credential)):  # noqa: E501
            raise ValueError(r"Invalid value for `credential`, must be a follow pattern or equal to `/^(?:[A-Za-z0-9+\/]{4})*(?:[A-Za-z0-9+\/]{2}==|[A-Za-z0-9+\/]{3}=)?$/`")  # noqa: E501

        self._credential = credential

    @property
    def deletion_timestamp(self) -> 'datetime':
        """Gets the deletion_timestamp of this V1Credential.  # noqa: E501


        :return: The deletion_timestamp of this V1Credential.  # noqa: E501
        :rtype: datetime
        """
        return self._deletion_timestamp

    @deletion_timestamp.setter
    def deletion_timestamp(self, deletion_timestamp: 'datetime'):
        """Sets the deletion_timestamp of this V1Credential.


        :param deletion_timestamp: The deletion_timestamp of this V1Credential.  # noqa: E501
        :type: datetime
        """

        self._deletion_timestamp = deletion_timestamp

    @property
    def id(self) -> 'str':
        """Gets the id of this V1Credential.  # noqa: E501


        :return: The id of this V1Credential.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id: 'str'):
        """Sets the id of this V1Credential.


        :param id: The id of this V1Credential.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def type(self) -> 'V1CredentialType':
        """Gets the type of this V1Credential.  # noqa: E501


        :return: The type of this V1Credential.  # noqa: E501
        :rtype: V1CredentialType
        """
        return self._type

    @type.setter
    def type(self, type: 'V1CredentialType'):
        """Sets the type of this V1Credential.


        :param type: The type of this V1Credential.  # noqa: E501
        :type: V1CredentialType
        """

        self._type = type

    @property
    def update_timestamp(self) -> 'datetime':
        """Gets the update_timestamp of this V1Credential.  # noqa: E501


        :return: The update_timestamp of this V1Credential.  # noqa: E501
        :rtype: datetime
        """
        return self._update_timestamp

    @update_timestamp.setter
    def update_timestamp(self, update_timestamp: 'datetime'):
        """Sets the update_timestamp of this V1Credential.


        :param update_timestamp: The update_timestamp of this V1Credential.  # noqa: E501
        :type: datetime
        """

        self._update_timestamp = update_timestamp

    @property
    def user_id(self) -> 'str':
        """Gets the user_id of this V1Credential.  # noqa: E501


        :return: The user_id of this V1Credential.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id: 'str'):
        """Sets the user_id of this V1Credential.


        :param user_id: The user_id of this V1Credential.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1Credential, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'V1Credential') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, V1Credential):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other: 'V1Credential') -> bool:
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1Credential):
            return True

        return self.to_dict() != other.to_dict()
